/*
 * Copyright (c) 2008 Nokia Corporation
 * Contact: integration@maemo.org
 *
 * Authors: Hallyson Melo <hallyson.melo@indt.org.br>
 *          Kyller Gorgonio <kyllercg@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <hildon/hildon.h>

#include "nm_bt_setup.h"
#include "nm_bt_utils.h"
#include "nm_bt_search_dialog.h"
#include "nm_ifaces_utils.h"
#include "nm_settings.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static void nm_bt_setup_defaults (NmBtSetup *, gboolean);
static void nm_bt_setup_create_widget(NmBtSetup *);
static void nm_bt_setup_check_initial_state (NmBtSetup *self);
static void nm_bt_setup_enable_cb (GtkToggleButton *, gpointer);
static void nm_bt_setup_enable_options (NmBtSetup *, gboolean);
static gboolean nm_bt_setup_search (GtkButton *, gpointer);

static gboolean nm_bt_setup_apply_changes (NmConnWidget *);
static gboolean nm_bt_setup_apply_changes_b (NmConnWidget *);

typedef struct _NmBtSetupPrivate NmBtSetupPrivate;

struct _NmBtSetupPrivate
{
    GtkWidget *checkbutton;

    GtkWidget *ip_label;
    GtkWidget *gw_label;
    GtkWidget *mask_label;

    GtkWidget *ip_entry;
    GtkWidget *gw_entry;
    GtkWidget *mask_entry;

    GtkWidget *btaddr_search;
    GtkWidget *btname_entry;
    GtkWidget *btaddr_frame;

    NmBtUtils *bt_utils;
    gchar *bt_addr;
};

#define NM_BT_SETUP_GET_PRIVATE(o) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_BT_SETUP_TYPE, NmBtSetupPrivate))

static void nm_bt_setup_class_init (NmBtSetupClass *klass);
static void nm_bt_setup_init       (NmBtSetup *self);
static void nm_bt_setup_dispose    (GObject *object);
static void nm_bt_setup_finalize   (GObject *object);

G_DEFINE_TYPE (NmBtSetup, nm_bt_setup, NM_CONN_WIDGET_TYPE)

static void
nm_bt_setup_class_init (NmBtSetupClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    NmConnWidgetClass *conn_widget_class = NM_CONN_WIDGET_CLASS (klass);

    g_type_class_add_private (klass, sizeof (NmBtSetupPrivate));

    object_class->dispose = nm_bt_setup_dispose;
    object_class->finalize = nm_bt_setup_finalize;

    conn_widget_class->apply_changes = nm_bt_setup_apply_changes;
    conn_widget_class->apply_changes_b = nm_bt_setup_apply_changes_b;
}

static void
nm_bt_setup_init (NmBtSetup *self)
{
    NmBtSetupPrivate *priv;

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    /* Init BT data, move to BT stuff later */
    priv->bt_addr = NULL;
    priv->bt_utils = nm_bt_utils_new ();

    nm_bt_setup_create_widget (self);
    nm_bt_setup_check_initial_state (self);
}

static void
nm_bt_setup_dispose (GObject *object)
{
    G_OBJECT_CLASS (nm_bt_setup_parent_class)->dispose (object);
}

static void
nm_bt_setup_finalize (GObject *object)
{
    NmBtSetupPrivate *priv;

    priv = NM_BT_SETUP_GET_PRIVATE (object);

    g_free (priv->bt_addr);
    g_object_unref (priv->bt_utils);

    G_OBJECT_CLASS (nm_bt_setup_parent_class)->finalize (object);
}

GtkWidget*
nm_bt_setup_new (void)
{
    return g_object_new (NM_BT_SETUP_TYPE, NULL);
}

void
nm_bt_setup_enable (NmBtSetup *self, gboolean enable)
{
    NmBtSetupPrivate *priv;

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(priv->checkbutton), enable);
}

gboolean
nm_bt_setup_is_enabled (NmBtSetup *self)
{
    NmBtSetupPrivate *priv;

    priv = NM_BT_SETUP_GET_PRIVATE (self);
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(priv->checkbutton));
}

static void
nm_bt_setup_create_widget(NmBtSetup *self)
{
    GtkWidget *table;
    GtkWidget *hbox;
    NmBtSetupPrivate *priv;

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    table = gtk_table_new (6, 2, TRUE);
    hbox = gtk_hbox_new (FALSE, 15);

    priv->checkbutton = gtk_check_button_new_with_label ("enable");
    g_signal_connect (priv->checkbutton, "toggled", G_CALLBACK(nm_bt_setup_enable_cb),
                      self);

    /* IP address */
    priv->ip_label = gtk_label_new("IP address: ");
    gtk_misc_set_alignment(GTK_MISC(priv->ip_label), 1, 0.5);
    priv->ip_entry = gtk_entry_new();

    /* Gateway */
    priv->gw_label = gtk_label_new("Gateway: ");
    gtk_misc_set_alignment(GTK_MISC(priv->gw_label), 1, 0.5);
    priv->gw_entry = gtk_entry_new();

    /* Network mask */
    priv->mask_label = gtk_label_new("Network mask: ");
    gtk_misc_set_alignment(GTK_MISC(priv->mask_label), 1, 0.5);
    priv->mask_entry = gtk_entry_new();

    /* Bluetooth address */
    priv->btaddr_frame = gtk_frame_new (" Bluetooth address ");
    priv->btaddr_search = gtk_button_new_with_label ("Search");
    g_signal_connect (priv->btaddr_search, "clicked",
                      G_CALLBACK (nm_bt_setup_search), self);

    priv->btname_entry = gtk_entry_new ();
    gtk_entry_set_editable (GTK_ENTRY (priv->btname_entry), FALSE);

    gtk_table_attach_defaults (GTK_TABLE (table), priv->checkbutton, 0, 1, 0, 1);
    gtk_table_attach_defaults (GTK_TABLE (table), priv->ip_label, 0, 1, 1, 2);
    gtk_table_attach_defaults (GTK_TABLE (table), priv->ip_entry, 1, 2, 1, 2);
    gtk_table_attach_defaults (GTK_TABLE (table), priv->gw_label, 0, 1, 2, 3);
    gtk_table_attach_defaults (GTK_TABLE (table), priv->gw_entry, 1, 2, 2, 3);
    gtk_table_attach_defaults (GTK_TABLE (table), priv->mask_label, 0, 1, 3, 4);
    gtk_table_attach_defaults (GTK_TABLE (table), priv->mask_entry, 1, 2, 3, 4);

    gtk_box_pack_start (GTK_BOX (hbox), priv->btaddr_search, FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), priv->btname_entry, FALSE, TRUE, 0);
    gtk_container_add (GTK_CONTAINER (priv->btaddr_frame), hbox);

    gtk_table_attach_defaults (GTK_TABLE (table), priv->btaddr_frame, 0, 2, 4, 6);

    gtk_container_add(GTK_CONTAINER (self), table);
}

static void
nm_bt_setup_check_initial_state (NmBtSetup *self)
{
    NmBtSetupPrivate *priv;

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    if (nm_bt_utils_radio_is_enabled (priv->bt_utils) &&
        nm_bt_utils_has_network_connection (priv->bt_utils)) {

        gchar *name = nm_bt_utils_get_network_connection_end_point (priv->bt_utils);

        if (name) {

            gtk_entry_set_text (GTK_ENTRY (priv->btname_entry), name);
        }
        nm_bt_setup_enable (self, TRUE);
    } else {

        nm_bt_setup_enable (self, FALSE);
    }

    nm_bt_setup_enable_cb (GTK_TOGGLE_BUTTON(priv->checkbutton), self);
    nm_bt_setup_enable_options (self, FALSE);
}

static void
nm_bt_setup_defaults (NmBtSetup *self, gboolean pressed)
{
    NmBtSetupPrivate *priv;
    NmSettings *settings  = nm_settings_get_context();;

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    if (pressed) {

        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry),
                           nm_settings_get_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_IP));
        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry),
                           nm_settings_get_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_GATEWAY));
        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry),
                           nm_settings_get_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_MASK));
    } else {
        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry), "");
    }

}

static void
nm_bt_setup_enable_cb (GtkToggleButton *button, gpointer data)
{
    NmBtSetupPrivate *priv;
    gboolean pressed = gtk_toggle_button_get_active (button);

    priv = NM_BT_SETUP_GET_PRIVATE (data);
    gtk_widget_set_sensitive (priv->btaddr_frame, pressed);
    gtk_widget_set_sensitive (priv->btaddr_search, pressed);

    nm_bt_setup_defaults(data, pressed);

    // FIXME: remove this if the combo box is not needed for something else
    nm_bt_setup_enable_options (data, pressed);

    nm_conn_widget_settings_changed (data);
}

static void
nm_bt_setup_enable_options (NmBtSetup *self, gboolean enable)
{
    NmBtSetupPrivate *priv;
    priv = NM_BT_SETUP_GET_PRIVATE (self);

    gtk_widget_set_sensitive (priv->ip_label, enable);
    gtk_widget_set_sensitive (priv->ip_entry, enable);
    gtk_widget_set_sensitive (priv->gw_label, enable);
    gtk_widget_set_sensitive (priv->gw_entry, enable);
    gtk_widget_set_sensitive (priv->mask_label, enable);
    gtk_widget_set_sensitive (priv->mask_entry, enable);
}

static gboolean
nm_bt_setup_apply_changes (NmConnWidget *self)
{
    NmBtSetupPrivate *priv;
    NmBtSetup *btsetup = NM_BT_SETUP (self);
    gchar *pand = NULL;
    NmSettings *settings = nm_settings_get_context();

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    if (nm_bt_setup_is_enabled (btsetup)) {

        if (!nm_bt_utils_radio_is_enabled (priv->bt_utils)) {

            gboolean res = nm_bt_utils_radio_enable (priv->bt_utils, TRUE);
            if (!res) {

                hildon_banner_show_information (NULL, NULL,
                                                 "Error enabling BT radio");
                return FALSE;
            }
        }

        if ((pand = nm_bt_utils_check_pand (priv->bt_utils,
                                             priv->bt_addr)) == NULL) {

            hildon_banner_show_information (NULL, NULL,
                                             "Unable to connect. Check if PAND is running on host!");
            return FALSE;
        }

        if (nm_ifaces_utils_save_config_e (NM_IFACES_BT_ST,
                gtk_entry_get_text (GTK_ENTRY (priv->ip_entry)),
                gtk_entry_get_text (GTK_ENTRY (priv->mask_entry)),
                gtk_entry_get_text (GTK_ENTRY (priv->gw_entry))) == FALSE) {

                    exit (1);
        } else {

            nm_settings_set_boolean (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_ENABLE, TRUE ); // is enabled
            nm_settings_set_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_IP,      gtk_entry_get_text (GTK_ENTRY (priv->ip_entry)));
            nm_settings_set_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_GATEWAY, gtk_entry_get_text (GTK_ENTRY (priv->gw_entry)));
            nm_settings_set_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_MASK,    gtk_entry_get_text (GTK_ENTRY (priv->mask_entry)));
            nm_settings_set_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_MAC,     priv->bt_addr);
            nm_settings_set_value (settings, NM_BT_SETTINGS_GROUP, NM_SETTINGS_HOST,    gtk_entry_get_text (GTK_ENTRY (priv->btname_entry)));
            nm_settings_save(settings);
        }

        if (!nm_bt_utils_connect (priv->bt_utils, priv->bt_addr)) {

            hildon_banner_show_information (NULL, NULL,
                                             "Could not connect to PC");
            return FALSE;
        }

        hildon_banner_show_information(NULL, NULL, "Bluetooth network enabled");
    } else {

        if (nm_bt_utils_remove_connection (priv->bt_utils)) {

            hildon_banner_show_information(NULL, NULL,
                                            "Bluetooth network disabled");
        } else {

            return FALSE;
        }
    }

    return TRUE;
}

static gboolean
nm_bt_setup_apply_changes_b (NmConnWidget *self)
{
    NmBtSetupPrivate *priv;
    gchar *pand = NULL;

    priv = NM_BT_SETUP_GET_PRIVATE (self);

    if (!nm_bt_utils_radio_is_enabled (priv->bt_utils)) {

        gboolean res = nm_bt_utils_radio_enable (priv->bt_utils, TRUE);
        if (!res) {

            hildon_banner_show_information (NULL, NULL,
                                             "Error enabling BT radio");
            return FALSE;
        }
    }

    if (!nm_bt_utils_has_network_connection (priv->bt_utils)) {

        if (nm_bt_setup_search (GTK_BUTTON(priv->btaddr_search), self)) {

            if ((pand = nm_bt_utils_check_pand (priv->bt_utils,
                                                priv->bt_addr)) == NULL) {

                hildon_banner_show_information (NULL, NULL,
                                                    "Unable to connect. Check if PAND is running on host!");
                return FALSE;
            }

            if (!nm_bt_utils_connect (priv->bt_utils, priv->bt_addr)) {

                hildon_banner_show_information (NULL, NULL,
                                                    "Could not connect to PC");
                return FALSE;
            }

            hildon_banner_show_information(NULL, NULL, "Bluetooth network enabled");

        } else {

            hildon_banner_show_information(NULL, NULL, "Operation cancelled");
        }
    } else {

        if (nm_bt_utils_remove_connection (priv->bt_utils)) {

            hildon_banner_show_information(NULL, NULL,
                                            "Bluetooth network disabled");
        } else {

            return FALSE;
        }
    }

    return TRUE;
}

static gboolean
nm_bt_setup_search (GtkButton *self, gpointer data)
{
    GtkWidget *search_dialog;
    NmBtSetupPrivate *priv;
    gint ret;

    priv = NM_BT_SETUP_GET_PRIVATE (data);

    search_dialog = nm_bt_search_dialog_new (priv->bt_utils);

    g_idle_add ((GSourceFunc) nm_bt_search_dialog_start_discovery, search_dialog);

    ret = gtk_dialog_run (GTK_DIALOG (search_dialog));

    nm_bt_search_dialog_stop_discovery (NM_BT_SEARCH_DIALOG (search_dialog));

    if (ret == GTK_RESPONSE_OK) {
        gchar *addr, *name;

        addr = nm_bt_search_dialog_get_selected_addr (NM_BT_SEARCH_DIALOG (search_dialog));
        name = nm_bt_search_dialog_get_selected_name (NM_BT_SEARCH_DIALOG (search_dialog));

        if (name != NULL && addr != NULL) {
            gtk_entry_set_text (GTK_ENTRY(priv->btname_entry), name);

            g_debug ("[%s] User selected device: %s - %s", __FUNCTION__, name, addr);

            priv->bt_addr = addr;
            g_free (name);
        }
    } else {

        gtk_widget_destroy (search_dialog);
        return FALSE;
    }

    gtk_widget_destroy (search_dialog);
    return TRUE;
}

