/*
 * Copyright (c) 2008 Nokia Corporation
 * Contact: integration@maemo.org
 *
 * Authors: Hallyson Melo <hallyson.melo@indt.org.br>
 *          Kyller Gorgonio <kyller@dee.ufcg.edu.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <stdlib.h>
#include <hildon/hildon-banner.h>

#include "nm_usb_setup.h"
#include "nm_usb_utils.h"
#include "nm_ifaces_utils.h"
#include "nm_main_dialog.h"
#include "nm_settings.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static void nm_usb_setup_defaults (NmUsbSetup *, gboolean);
static void nm_usb_setup_create_widget (NmUsbSetup *);
static void nm_usb_setup_enable_cb (GtkToggleButton *, gpointer);
static void nm_usb_setup_enable_options (NmUsbSetup *, gboolean);

typedef struct _NmUsbSetupPrivate NmUsbSetupPrivate;

struct _NmUsbSetupPrivate
{
    GtkWidget *checkbutton;

    GtkWidget *ip_label;
    GtkWidget *gw_label;
    GtkWidget *mask_label;

    GtkWidget *ip_entry;
    GtkWidget *gw_entry;
    GtkWidget *mask_entry;
};

#define NM_USB_SETUP_GET_PRIVATE(o) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_USB_SETUP_TYPE, NmUsbSetupPrivate))

static void nm_usb_setup_class_init (NmUsbSetupClass *);
static void nm_usb_setup_init       (NmUsbSetup *);
static void nm_usb_setup_dispose    (GObject *);
static void nm_usb_setup_finalize   (GObject *);

static gboolean nm_usb_setup_apply_changes (NmConnWidget *);
static gboolean nm_usb_setup_apply_changes_b (NmConnWidget *);

G_DEFINE_TYPE (NmUsbSetup, nm_usb_setup, NM_CONN_WIDGET_TYPE)

static void
nm_usb_setup_class_init (NmUsbSetupClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    NmConnWidgetClass *conn_widget_class = NM_CONN_WIDGET_CLASS (klass);

    g_type_class_add_private (klass, sizeof (NmUsbSetupPrivate));

    object_class->dispose = nm_usb_setup_dispose;
    object_class->finalize = nm_usb_setup_finalize;

    conn_widget_class->apply_changes = nm_usb_setup_apply_changes;
    conn_widget_class->apply_changes_b = nm_usb_setup_apply_changes_b;
}

static void
nm_usb_setup_init (NmUsbSetup *self)
{
    nm_usb_setup_create_widget (self);
}

static void
nm_usb_setup_dispose (GObject *object)
{
    G_OBJECT_CLASS (nm_usb_setup_parent_class)->dispose (object);
}

static void
nm_usb_setup_finalize (GObject *object)
{
    G_OBJECT_CLASS (nm_usb_setup_parent_class)->finalize (object);
}

GtkWidget*
nm_usb_setup_new (void)
{
    return g_object_new (NM_USB_SETUP_TYPE, NULL);
}

void
nm_usb_setup_enable (NmUsbSetup *self, gboolean enable)
{
    NmUsbSetupPrivate *priv;

    priv = NM_USB_SETUP_GET_PRIVATE (self);

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(priv->checkbutton), enable);

}

gboolean
nm_usb_setup_is_enabled (NmUsbSetup *self)
{
    NmUsbSetupPrivate *priv;

    priv = NM_USB_SETUP_GET_PRIVATE (self);
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(priv->checkbutton));
}

static void
nm_usb_setup_create_widget (NmUsbSetup *self)
{
    GtkWidget *table;
    NmUsbSetupPrivate *priv;

    priv = NM_USB_SETUP_GET_PRIVATE (self);

    table = gtk_table_new (4, 2, TRUE);

    priv->checkbutton = gtk_check_button_new_with_label ("enable");
    g_signal_connect (priv->checkbutton, "toggled", G_CALLBACK(nm_usb_setup_enable_cb),
                      self);

    /* IP address */
    priv->ip_label = gtk_label_new("IP address: ");
    gtk_misc_set_alignment(GTK_MISC(priv->ip_label), 1, 0.5);
    priv->ip_entry = gtk_entry_new();

    /* Gateway */
    priv->gw_label = gtk_label_new("Gateway: ");
    gtk_misc_set_alignment(GTK_MISC(priv->gw_label), 1, 0.5);
    priv->gw_entry = gtk_entry_new();

    /* Network mask */
    priv->mask_label = gtk_label_new("Network mask: ");
    gtk_misc_set_alignment(GTK_MISC(priv->mask_label), 1, 0.5);
    priv->mask_entry = gtk_entry_new();

    gtk_table_attach_defaults(GTK_TABLE (table), priv->checkbutton, 0, 1, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE (table), priv->ip_label, 0, 1, 1, 2);
    gtk_table_attach_defaults(GTK_TABLE (table), priv->ip_entry, 1, 2, 1, 2);
    gtk_table_attach_defaults(GTK_TABLE (table), priv->gw_label, 0, 1, 2, 3);
    gtk_table_attach_defaults(GTK_TABLE (table), priv->gw_entry, 1, 2, 2, 3);
    gtk_table_attach_defaults(GTK_TABLE (table), priv->mask_label, 0, 1, 3, 4);
    gtk_table_attach_defaults(GTK_TABLE( table), priv->mask_entry, 1, 2, 3, 4);

    gtk_container_add(GTK_CONTAINER (self), table);

    nm_usb_setup_enable_options (self, FALSE);
    if (nm_usb_utils_is_loaded ()) {
        nm_usb_setup_enable (self, TRUE);
        nm_usb_setup_enable_cb (GTK_TOGGLE_BUTTON (priv->checkbutton), self);
    } else {
        nm_usb_setup_enable_cb (GTK_TOGGLE_BUTTON (priv->checkbutton), self);
    }
}

static void
nm_usb_setup_defaults (NmUsbSetup *self, gboolean pressed)
{
    NmUsbSetupPrivate *priv;
    NmSettings *settings;

    priv = NM_USB_SETUP_GET_PRIVATE (self);
    settings = nm_settings_get_context();

    if (pressed) {
        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry),
                           nm_settings_get_value (settings, NM_USB_SETTINGS_GROUP, NM_SETTINGS_IP));

        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry),
                           nm_settings_get_value (settings, NM_USB_SETTINGS_GROUP, NM_SETTINGS_GATEWAY));
        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry),
                           nm_settings_get_value (settings, NM_USB_SETTINGS_GROUP, NM_SETTINGS_MASK));
    } else {
        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry), "");
    }
}

static void
nm_usb_setup_enable_cb (GtkToggleButton *button, gpointer data)
{
    NmUsbSetupPrivate *priv;
    gboolean pressed = gtk_toggle_button_get_active (button);

    priv = NM_USB_SETUP_GET_PRIVATE (data);

    nm_usb_setup_defaults (data, pressed);

    // FIXME: remove this if the combo box is needed for something else
    nm_usb_setup_enable_options (data, pressed);

    nm_conn_widget_settings_changed (data);
}

static void
nm_usb_setup_enable_options (NmUsbSetup *self, gboolean enable)
{
    NmUsbSetupPrivate *priv;
    priv = NM_USB_SETUP_GET_PRIVATE (self);

    gtk_widget_set_sensitive (priv->ip_label, enable);
    gtk_widget_set_sensitive (priv->ip_entry, enable);
    gtk_widget_set_sensitive (priv->gw_label, enable);
    gtk_widget_set_sensitive (priv->gw_entry, enable);
    gtk_widget_set_sensitive (priv->mask_label, enable);
    gtk_widget_set_sensitive (priv->mask_entry, enable);
}

static gboolean
nm_usb_setup_apply_changes (NmConnWidget *self)
{
    NmUsbSetup *usbsetup = NM_USB_SETUP (self);
    gboolean is_loaded = nm_usb_utils_is_loaded();
    NmSettings *settings = nm_settings_get_context();

    NmUsbSetupPrivate *priv;
    priv = NM_USB_SETUP_GET_PRIVATE (usbsetup);

    if (nm_usb_setup_is_enabled (usbsetup)) {

        if (!is_loaded) {

            if (nm_ifaces_utils_save_config_e (NM_IFACES_USB_ST,
                gtk_entry_get_text (GTK_ENTRY (priv->ip_entry)),
                gtk_entry_get_text (GTK_ENTRY (priv->mask_entry)),
                gtk_entry_get_text (GTK_ENTRY (priv->gw_entry))) == FALSE) {

                exit (1);
            } else {

                nm_settings_set_boolean (settings, NM_USB_SETTINGS_GROUP,
                                          NM_SETTINGS_ENABLE, TRUE ); // is enabled
                nm_settings_set_value (settings, NM_USB_SETTINGS_GROUP, NM_SETTINGS_IP, gtk_entry_get_text (GTK_ENTRY (priv->ip_entry)));
                nm_settings_set_value (settings, NM_USB_SETTINGS_GROUP, NM_SETTINGS_GATEWAY, gtk_entry_get_text (GTK_ENTRY (priv->gw_entry)));
                nm_settings_set_value (settings, NM_USB_SETTINGS_GROUP, NM_SETTINGS_MASK, gtk_entry_get_text (GTK_ENTRY (priv->mask_entry)));
                nm_settings_save(settings);
            }

            if (nm_usb_utils_load ()) {

                hildon_banner_show_information(NULL, NULL,
                                               "USB network enabled");
            } else {

                return FALSE;
            }
        }
    } else if (is_loaded) {

        if (nm_usb_utils_unload ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "USB file storage enabled");
        } else {

            return FALSE;
        }
    }

    return TRUE;
}

static gboolean
nm_usb_setup_apply_changes_b (NmConnWidget *self)
{
    NmUsbSetup *usbsetup = NM_USB_SETUP (self);
    gboolean is_loaded = nm_usb_utils_is_loaded();

    NmUsbSetupPrivate *priv;
    priv = NM_USB_SETUP_GET_PRIVATE (usbsetup);

    if (!is_loaded) {

        if (nm_usb_utils_load ()) {

            hildon_banner_show_information(NULL, NULL, "USB network enabled");
        } else {

            return FALSE;
        }
    } else if (is_loaded) {

        if (nm_usb_utils_unload ()) {

            hildon_banner_show_information(NULL, NULL,
                                           "USB file storage enabled");
        } else {

            return FALSE;
        }
    }

    return TRUE;
}

