/*
 * Copyright (c) 2008 Nokia Corporation
 * Contact: integration@maemo.org
 *
 * Authors: Marcos Morais <morais@dee.ufcg.edu.br>
 *          Kyller Gorgonio <kyllercg@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <gconf/gconf-client.h>
#include <hildon/hildon-banner.h>

#include "nm_wlan_setup.h"
#include "nm_wlan_utils.h"
#include "nm_ifaces_utils.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static void nm_wlan_setup_defaults (NmWlanSetup *, gboolean);
static void nm_wlan_setup_create_widget (NmWlanSetup *);
static void nm_wlan_setup_enable_cb (GtkToggleButton *, gpointer);
static void nm_wlan_setup_enable_options (NmWlanSetup *, gboolean);

typedef struct _NmWlanSetupPrivate NmWlanSetupPrivate;

struct _NmWlanSetupPrivate
{
    GtkWidget *checkbutton;

    GtkWidget *ip_label;
    GtkWidget *gw_label;
    GtkWidget *mask_label;
    GtkWidget *essid_label;

    GtkWidget *ip_entry;
    GtkWidget *gw_entry;
    GtkWidget *mask_entry;
    GtkWidget *essid_entry;

    NmWlanUtils *wlan_utils;
};

#define NM_WLAN_SETUP_GET_PRIVATE(o) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_WLAN_SETUP_TYPE, NmWlanSetupPrivate))

static void nm_wlan_setup_class_init (NmWlanSetupClass *);
static void nm_wlan_setup_init       (NmWlanSetup *);
static void nm_wlan_setup_dispose    (GObject *);
static void nm_wlan_setup_finalize   (GObject *);

static gboolean nm_wlan_setup_apply_changes (NmConnWidget *);
static gboolean nm_wlan_setup_apply_changes_b (NmConnWidget *);

G_DEFINE_TYPE (NmWlanSetup, nm_wlan_setup, NM_CONN_WIDGET_TYPE)

static void
nm_wlan_setup_class_init (NmWlanSetupClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);
    NmConnWidgetClass *conn_widget_class = NM_CONN_WIDGET_CLASS (klass);

    g_type_class_add_private (klass, sizeof (NmWlanSetupPrivate));

    object_class->dispose = nm_wlan_setup_dispose;
    object_class->finalize = nm_wlan_setup_finalize;

    conn_widget_class->apply_changes = nm_wlan_setup_apply_changes;
    conn_widget_class->apply_changes_b = nm_wlan_setup_apply_changes_b;
}

static void
nm_wlan_setup_init (NmWlanSetup *self)
{
    NmWlanSetupPrivate *priv;

    priv = NM_WLAN_SETUP_GET_PRIVATE (self);
    priv->wlan_utils = nm_wlan_utils_new ();

    nm_wlan_setup_create_widget (self);
}

static void
nm_wlan_setup_dispose (GObject *object)
{
    G_OBJECT_CLASS (nm_wlan_setup_parent_class)->dispose (object);
}

static void
nm_wlan_setup_finalize (GObject *object)
{
    G_OBJECT_CLASS (nm_wlan_setup_parent_class)->finalize (object);
}

GtkWidget*
nm_wlan_setup_new (void)
{
    return g_object_new (NM_WLAN_SETUP_TYPE, NULL);
}

void
nm_wlan_setup_enable (NmWlanSetup *self, gboolean enable)
{
    NmWlanSetupPrivate *priv;

    priv = NM_WLAN_SETUP_GET_PRIVATE (self);

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(priv->checkbutton), enable);
}

gboolean
nm_wlan_setup_is_enabled (NmWlanSetup *self)
{
    NmWlanSetupPrivate *priv;

    priv = NM_WLAN_SETUP_GET_PRIVATE (self);
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(priv->checkbutton));
}

static void
nm_wlan_setup_create_widget(NmWlanSetup *self)
{
    GtkWidget *table;
    NmWlanSetupPrivate *priv;

    priv = NM_WLAN_SETUP_GET_PRIVATE (self);

    table = gtk_table_new (5, 2, TRUE);

    priv->checkbutton = gtk_check_button_new_with_label ("enable");
    g_signal_connect (priv->checkbutton, "toggled", G_CALLBACK(nm_wlan_setup_enable_cb),
                      self);

    /* IP address */
    priv->ip_label = gtk_label_new("IP address: ");
    gtk_misc_set_alignment(GTK_MISC(priv->ip_label), 1, 0.5);
    priv->ip_entry = gtk_entry_new();

    /* Gateway */
    priv->gw_label = gtk_label_new("Gateway: ");
    gtk_misc_set_alignment(GTK_MISC(priv->gw_label), 1, 0.5);
    priv->gw_entry = gtk_entry_new();

    /* Network mask */
    priv->mask_label = gtk_label_new("Network mask: ");
    gtk_misc_set_alignment(GTK_MISC(priv->mask_label), 1, 0.5);
    priv->mask_entry = gtk_entry_new();

    /* ESSID */
    priv->essid_label = gtk_label_new("ESSID: ");
    gtk_misc_set_alignment(GTK_MISC(priv->essid_label), 1, 0.5);
    priv->essid_entry = gtk_entry_new();

    gtk_table_attach_defaults(GTK_TABLE(table), priv->checkbutton, 0, 1, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->ip_label, 0, 1, 1, 2);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->ip_entry, 1, 2, 1, 2);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->gw_label, 0, 1, 2, 3);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->gw_entry, 1, 2, 2, 3);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->mask_label, 0, 1, 3, 4);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->mask_entry, 1, 2, 3, 4);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->essid_label, 0, 1, 4, 5);
    gtk_table_attach_defaults(GTK_TABLE(table), priv->essid_entry, 1, 2, 4, 5);

    gtk_container_add(GTK_CONTAINER(self), table);

    nm_wlan_setup_enable_options (self, FALSE);
    if (nm_wlan_utils_is_active ()) {
        nm_wlan_setup_enable (self, TRUE);
        nm_wlan_setup_enable_cb (GTK_TOGGLE_BUTTON(priv->checkbutton), self);
    } else {
        nm_wlan_setup_enable_cb (GTK_TOGGLE_BUTTON(priv->checkbutton), self);
    }

}

static void
nm_wlan_setup_defaults (NmWlanSetup *self, gboolean pressed)
{
    NmWlanSetupPrivate *priv;
    gchar* value = NULL;
    GSList *essid_list, *l;
    gint i = 0;
    GConfClient* gcClient = NULL;


    gcClient = gconf_client_get_default();
    g_assert(GCONF_IS_CLIENT(gcClient));

    priv = NM_WLAN_SETUP_GET_PRIVATE (self);

    if (pressed) {

        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_TYPE, NM_WLAN_DEFAULT_TYPE , NULL);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_IPV4_TYPE, NM_WLAN_DEFAULT_IPV4_TYPE , NULL);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_HIDDEN, NM_WLAN_DEFAULT_HIDDEN , NULL);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_NAME, NM_WLAN_DEFAULT_NAME , NULL);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_SECURITY, NM_WLAN_DEFAULT_SECURITY , NULL);

        value = gconf_client_get_string(gcClient, NM_GCONF_KEY_WLAN_IP , NULL);
        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry), value == NULL ? NM_WLAN_DEFAULT_IP : value);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_IP , value == NULL ? NM_WLAN_DEFAULT_IP : value, NULL);

        value = gconf_client_get_string(gcClient, NM_GCONF_KEY_WLAN_GATEWAY, NULL);
        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry), value == NULL ? NM_WLAN_DEFAULT_GATEWAY : value);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_GATEWAY, value == NULL ? NM_WLAN_DEFAULT_GATEWAY : value, NULL);

        value = gconf_client_get_string(gcClient, NM_GCONF_KEY_WLAN_MASK, NULL);
        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry), value == NULL ? NM_WLAN_DEFAULT_MASK : value);
        gconf_client_set_string(gcClient, NM_GCONF_KEY_WLAN_MASK, value == NULL ? NM_WLAN_DEFAULT_MASK : value, NULL);

        essid_list = gconf_client_get_list (gcClient, NM_GCONF_KEY_WLAN_ESSID, GCONF_VALUE_INT, NULL);

        if (g_slist_length(essid_list) > 0) {

            /* the ESSID value is stored as a list of integers in gconf */
            value = g_malloc(50);
            for (l = essid_list, i=0; l != NULL && i < 50; l = l->next, i++)
                value[i] = (gchar)(GPOINTER_TO_INT(l->data));
            value[i]='\0';

            gtk_entry_set_text(GTK_ENTRY (priv->essid_entry), value);

            g_free(value);
        } else {

            gtk_entry_set_text(GTK_ENTRY (priv->essid_entry), NM_WLAN_DEFAULT_ESSID);
            value = g_strdup(NM_WLAN_DEFAULT_ESSID);
            /* the ESSID value is stored as a list of integers in gconf */
            for (i = 0; value[i] && i < 50; i++)
                essid_list = g_slist_append (essid_list, GINT_TO_POINTER( (gint) (value[i]) ));

            gconf_client_set_list(gcClient, NM_GCONF_KEY_WLAN_ESSID, GCONF_VALUE_INT, essid_list, NULL);
        }

    } else {

        gtk_entry_set_text(GTK_ENTRY (priv->ip_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->gw_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->mask_entry), "");
        gtk_entry_set_text(GTK_ENTRY (priv->essid_entry), "");
    }
}

static void
nm_wlan_setup_enable_cb (GtkToggleButton *button, gpointer data)
{
    NmWlanSetupPrivate *priv;
    gboolean pressed = gtk_toggle_button_get_active (button);

    priv = NM_WLAN_SETUP_GET_PRIVATE (data);

    nm_wlan_setup_defaults(data, pressed);

    // FIXME: remove this if the combo box is needed for something else
    nm_wlan_setup_enable_options (data, pressed);

    nm_conn_widget_settings_changed (data);
}

static void
nm_wlan_setup_enable_options (NmWlanSetup *self, gboolean enable)
{
    NmWlanSetupPrivate *priv;
    priv = NM_WLAN_SETUP_GET_PRIVATE (self);

    gtk_widget_set_sensitive (priv->ip_label, enable);
    gtk_widget_set_sensitive (priv->ip_entry, enable);
    gtk_widget_set_sensitive (priv->gw_label, enable);
    gtk_widget_set_sensitive (priv->gw_entry, enable);
    gtk_widget_set_sensitive (priv->mask_label, enable);
    gtk_widget_set_sensitive (priv->mask_entry, enable);
    gtk_widget_set_sensitive (priv->essid_label, enable);
    gtk_widget_set_sensitive (priv->essid_entry, enable);
}

static gboolean
nm_wlan_setup_apply_changes (NmConnWidget *self)
{
    NmWlanSetup *wlansetup = NM_WLAN_SETUP (self);
    NmWlanSetupPrivate *priv;
    priv = NM_WLAN_SETUP_GET_PRIVATE (wlansetup);

    const gchar* value = NULL;
    GSList *essid_list = NULL;
    gint i = 0;
    GConfClient* gcClient = NULL;

    gcClient = gconf_client_get_default ();
    g_assert (GCONF_IS_CLIENT (gcClient));

    gboolean is_active = nm_wlan_utils_is_active ();

    if (nm_wlan_setup_is_enabled (wlansetup)) {

        if (!is_active) {

            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_TYPE, NM_WLAN_DEFAULT_TYPE , NULL);
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_IPV4_TYPE, NM_WLAN_DEFAULT_IPV4_TYPE , NULL);
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_HIDDEN, NM_WLAN_DEFAULT_HIDDEN , NULL);
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_NAME, NM_WLAN_DEFAULT_NAME , NULL);
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_SECURITY, NM_WLAN_DEFAULT_SECURITY , NULL);

            value = gtk_entry_get_text (GTK_ENTRY (priv->ip_entry));
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_IP , value, NULL);

            value = gtk_entry_get_text (GTK_ENTRY (priv->gw_entry));
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_GATEWAY, value, NULL);

            value = gtk_entry_get_text (GTK_ENTRY (priv->mask_entry));
            gconf_client_set_string (gcClient, NM_GCONF_KEY_WLAN_MASK, value, NULL);

            value = gtk_entry_get_text (GTK_ENTRY (priv->essid_entry));

            /* the ESSID value is stored as a list of integers in gconf */
            for (i = 0; value[i] && i < 50; i++)
                essid_list = g_slist_append (essid_list, GINT_TO_POINTER ((gint) (value[i])));

            gconf_client_set_list (gcClient, NM_GCONF_KEY_WLAN_ESSID, GCONF_VALUE_INT, essid_list, NULL);

            if (nm_wlan_utils_activate ()) {

                hildon_banner_show_information (NULL, NULL, "WLAN Ad-Hoc network enabled");
            } else {

                hildon_banner_show_information (NULL, NULL, "Error activating WLAN Ad-Hoc network");
                return FALSE;
            }
        }
    } else { //if (is_active) {

        if (nm_wlan_utils_deactivate ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "WLAN Ad-Hoc network disabled");
        } else {

            return FALSE;
        }
    }

    return TRUE;
}

static gboolean
nm_wlan_setup_apply_changes_b (NmConnWidget *self)
{
    NmWlanSetup *wlansetup = NM_WLAN_SETUP (self);
    NmWlanSetupPrivate *priv;
    priv = NM_WLAN_SETUP_GET_PRIVATE (wlansetup);

    gboolean is_active = nm_wlan_utils_is_active ();

    if (!is_active) {

        if (nm_wlan_utils_activate ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "WLAN Ad-Hoc network enabled");
        } else {

            return FALSE;
        }
    } else { //if (is_active) {

        if (nm_wlan_utils_deactivate ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "WLAN Ad-Hoc network disabled");

        } else {

            return FALSE;
        }
    }

    return TRUE;
}

